const path = require('path')
const childProcess = require('child_process')
const phantomjs = require('phantomjs')
const binPath = phantomjs.path
const through = require('through2')
const Either = require('data.either')
const Task = require('data.task')
let output = '';

const parse = Either.try(JSON.parse)

const cleanStyleResult = style => {
  style.forEach(x => {
    if(x) {
      Object.keys(x).forEach(k => {
        if(parseFloat(k) >= 0) delete x[k]
      })
      delete x.cssText
    }
  })
  return style
}

const prepareContents = file =>
  parse(file)
  .map(o => Object.assign(o, {style: cleanStyleResult(o.style)}))

const outputToSnapContents = output =>
  output
  .split('\n')
  .filter(s => s.trim())
  .reduce((acc, x) =>
    prepareContents(x)
    .map(y => Object.assign(acc, {[y.name]: y}))
    .getOrElse(acc), {})

const report = () => {
  const transform = (file, enc, next) => {
    output += String(file)
    next(null, null);
  };

  const flush = function(next){
    this.push(output)
    next(null, output)
  }

  return through(transform, flush);
};

const runPhantom = (folder, stylePath) => {
  const childArgs = [ path.join(__dirname, 'phantomIt.js'), folder, stylePath ]
  const child = childProcess.execFile(binPath, childArgs)
  child.stdout.removeAllListeners("data");
  child.stderr.removeAllListeners("data");
  return child.stdout
}

const getStyleOutput = (folder, stylePath) =>
  new Task((rej, res) => {
    runPhantom(folder, stylePath)
    .pipe(report())
    .on('finish', () => res(output))
    .on('error', rej)
  })

module.exports = (folder, stylePath) =>
  getStyleOutput(folder, stylePath)
  .map(outputToSnapContents)

